/*
 Autor: Jose Alberto Gandullo Ávila
 Proyecto: SIMOCAP
 Blog: http://simocap.wordpress.com
 correo: yoalberto007@gmail.com
 fecha: 2010-06-08
 */

#ifdef _WIN32
#  include <windows.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#ifndef __APPLE__
#  include <GL/glut.h>
#else
#  include <GLUT/glut.h>
#endif
#include <AR/gsub.h>
#include <AR/param.h>
#include <AR/ar.h>
#include <AR/video.h>

#include "object.h"

#define COLLIDE_DIST 30000.0

#define PI         3.14159265358979323846

// estructura para guardar los 3 angulos de un hueso
typedef struct {
	double x;
	double y;
	double z;
} Bone;

typedef struct {
	Bone head;
	Bone arm_right;
	Bone forearm_right;
} Frame;

typedef struct {
	Bone top;
	Bone front;
	Bone botton;
	Bone back;
} Cube;

Frame lastFrame;
Cube r_arm;
//nombre del archivo de salida
char outbvh[255];
//lineas de movimiento del archivo bvh de salida
int cuenta_lineas = 0;
//cadena para dar valor a la variable de entorno, 
//antes export ARTOOLKIT_CONFIG
/*char
 *varEntConf =
 "ARTOOLKIT_CONFIG= filesrc location=/home/alberto/Escritorio/proyectos/videos/brazo3.MPG ! decodebin ! ffmpegcolorspace ! capsfilter caps=video/x-raw-rgb,bpp=24 ! identity name=artoolkit ! fakesink";
 */
char varEntConf[500];
//variable que indica la opcion de captura seleccionada
int captureOption;

int comenzarCaptura = 0;
// variables globales??
/* Object Data */
char model_name[500] = "Data/patt/object_data";
ObjectData_T *object;
int objectnum;

int xsize, ysize;
int thresh = 150;
int count = 0;

/* set up the video format globals */

#ifdef _WIN32
char *vconf = "Data\\WDM_camera_flipV.xml";
#else
char *vconf = "";
#endif

char *cparam_name = "Data/camera_para.dat";
ARParam cparam;

static void init(void);
static void cleanup(void);
static void keyEvent(unsigned char key, int x, int y);
static void mainLoop(void);
static int draw(ObjectData_T *object, int objectnum);
static int draw_object(int obj_id, double gl_para[16]);
static void draw_axis(void);

int matrizConversion(double mat[3][4], double rot[3][3]);
int obtenerAngulos(double rot[3][3], double *angx, double *angy, double *angz);
int obtenerAngulosRadianes(double rot[3][3], double *angx, double *angy,
		double *angz);
int multiplicarMatrices(double m[3][3], double n[3][3], double r[3][3]);
int imprimirMatriz(double m[3][3]);
int angulosToMatriz(double ax, double ay, double az, double rot[3][3]);
int imprimirMatrizOpengl(GLfloat m[16]);
int getAngulos(double rot[3][3], double *rfXAngle, double *rfYAngle,
		double *rfZAngle);
int copiarBaseBvh(void);

int printFrame(void);
int headMotion(ObjectData_T base, ObjectData_T head_front,
		ObjectData_T head_right, ObjectData_T head_left, Bone *head);
int armRightMotion(ObjectData_T base, ObjectData_T arm_right_front,
		ObjectData_T arm_right_top, ObjectData_T arm_right_botton,
		ObjectData_T arm_right_back, Bone *arm_right);
int forearmRightMotion(ObjectData_T arm_right_front,
		ObjectData_T arm_right_top, ObjectData_T arm_right_botton,
		ObjectData_T arm_right_back, ObjectData_T forearm_right_front,
		ObjectData_T forearm_right_top, ObjectData_T forearm_right_botton,
		ObjectData_T forearm_right_back, Bone *forearm_right);

void print_string( char *string );
void print_string_autor( char *string );

/* esta funcion nos pide que introduzcamos por teclado la ruta del video a procesar
 * y completa con esta ruta la cadena de caracteres para la variable de entorno de 
 * configuracion
 */
int localizacion(char config[]) {
	int outscanf;
	char localizacion[500];
	printf("Introduzca la ruta del archivo de video a procesar.\n ");
	printf("Ejemplo : '/home/videos/video.MPG' (sin las comillas)\n");

	outscanf = scanf("%[^\n]s", localizacion);
	//printf("%s \n",localizacion);
	sprintf(
			config,
			"ARTOOLKIT_CONFIG= filesrc location=%s ! decodebin ! ffmpegcolorspace ! capsfilter caps=video/x-raw-rgb,bpp=24 ! identity name=artoolkit ! fakesink",
			localizacion);
}

int opciones(char model[]) {
	int outscanf;
	int opcion;
	printf("Seleccione entre una de las siguientes opciones:\n ");
	printf(" 	1.- Combinado (las dos articulaciones a la vez)\n");
	printf(" 	2.- Capturar articulación del CUELLO (movimiento cabeza)\n");
	printf(
			" 	3.- Capturar articulación del HOMBRO (movimiento brazo derecho)\n");

	getchar(); //arreglo para que funcione el scanf siguiente
	outscanf = scanf("%d", &opcion);
	printf("Elegida opcion %d \n", opcion);

	if (opcion < 1 || opcion > 3) {
		return 0;
	} else if (opcion == 1) {
		sprintf(model, "Data/patt/object_data");
		return 1;
	} else if (opcion == 2) {
		sprintf(model, "Data/patt/object_data_cabeza");
		return 2;
	} else if (opcion == 3) {
		sprintf(model, "Data/patt/object_data");
		return 3;
	}

}

static void init(void) {
	ARParam wparam;

	/* open the video path */
	if (arVideoOpen(vconf) < 0)
		exit(0);
	/* find the size of the window */
	if (arVideoInqSize(&xsize, &ysize) < 0)
		exit(0);
	printf("Image size (x,y) = (%d,%d)\n", xsize, ysize);

	/* set the initial camera parameters */
	if (arParamLoad(cparam_name, 1, &wparam) < 0) {
		printf("Camera parameter load error !!\n");
		exit(0);
	}
	arParamChangeSize(&wparam, xsize, ysize, &cparam);
	arInitCparam(&cparam);
	printf("*** Camera Parameter ***\n");
	arParamDisp(&cparam);

	/* load in the object data - trained markers and associated bitmap files */
	if ((object = read_ObjData(model_name, &objectnum)) == NULL) {
		printf("Datos de objeto no cargados del archivo %s\n", model_name);
		exit(0);
	}
	printf("Objectfile num = %d\n", objectnum);

	/* open the graphics window */
	argInit(&cparam, 1.0, 0, 0, 0, 0);
}

int main(int argc, char **argv) {

	//Bienvenida y Autoría
	printf("Bienvenido a SIMOCAP. Ultima actualización en Junio 2010\n ");
	printf("Autor: 	José Alberto Gandullo Ávila. \n ");
	printf("Blog:	http://simocap.wordpress.com \n ");
	printf("Correo: yoalberto007@gmail.com \n \n");
	
	//pido la ruta de video para completar la configuracion
	localizacion(varEntConf);

	//permite seleccionar la opcion de captura mas adecuada
	captureOption = opciones(model_name);

	printf("Variable de entorno %s \n", varEntConf);
	//le doy mi configuración a mi variable de entorno
	putenv(varEntConf);

	//copiar archivo base.bvh a archivo de salida
	copyFile();
	//initialize applications
	glutInit(&argc, argv);
	init();

	arVideoCapStart();

	//abrir archivo de salida

	//start the main event loop
	argMainLoop(NULL, keyEvent, mainLoop);//dentro de mainLoop agregar lineas a archivo de salida

	printf("numero de Lineas %d \n", cuenta_lineas);
	//cerrar archivo de salida

	return 0;
}

static void keyEvent(unsigned char key, int x, int y) {

	/* START CAPTURE IF SPACE key is pressed */
		if (key == 0x20) {
			comenzarCaptura = 1;
		}
	
	/* quit if the ESC key is pressed */
	if (key == 0x1b) {
		cleanup();
		printf("*** %f (frame/sec)\n", (double) count / arUtilTimer());
		printf("numero de Lineas %d \n", cuenta_lineas);
		postprocesado();
			printf("\n SIMOCAP. Junio 2010\n ");
			printf("Autor: 	José Alberto Gandullo Ávila. \n ");
			printf("Blog:	http://simocap.wordpress.com \n ");
			printf("Correo: yoalberto007@gmail.com \n \n");
		
		exit(0);
	}
}

/* main loop */
static void mainLoop(void) {
	double rot[3][3], rot90y[3][4], res[3][3]; // matriz de rotacion
	double cp;
	double a, b, c;
	double cabezaX, cabezaY, cabezaZ;
	double ax, ay, az;
	double quat[4], pos[3];
	ARUint8 *dataPtr;
	ARMarkerInfo *marker_info;
	int marker_num;
	int i, j, k;
	double wmat1[3][4], wmat2[3][4];
	FILE *f;
	int imprimir;
	Bone head, arm_right, forearm_right;
	char mensaje1[255] = "Para comenzar la captura pulse 'ESPACIO'";
	char mensaje2[255] = "Para finalizar la captura pulse 'ESCAPE'";
	char autor[255] = "SIMOCAP by Alberto Gandullo, yoalberto007@gmail.com";
	
	/* grab a video frame */
	if ((dataPtr = (ARUint8 *) arVideoGetImage()) == NULL) {
		arUtilSleep(2);
		return;
	}

	if (count == 0)
		arUtilTimerReset();
	count++;

	/*draw the video*/
	argDrawMode2D();
	argDispImage(dataPtr, 0, 0);

	glColor3f(1.0, 0.0, 0.0);
	glLineWidth(6.0);

	/* detect the markers in the video frame */
	if (arDetectMarker(dataPtr, thresh, &marker_info, &marker_num) < 0) {
		cleanup();
		exit(0);
	}
	for (i = 0; i < marker_num; i++) {
		//argDrawSquare(marker_info[i].vertex,0,0);
	}

	/* check for known patterns */
	for (i = 0; i < objectnum; i++) {
		k = -1;
		for (j = 0; j < marker_num; j++) {
			if (object[i].id == marker_info[j].id) {

				/* you've found a pattern */
				//printf("Found pattern: %d ",patt_id);
				glColor3f(0.0, 1.0, 0.0);
				argDrawSquare(marker_info[j].vertex, 0, 0);

				if (k == -1)
					k = j;
				else /* make sure you have the best pattern (highest confidence factor) */
				if (marker_info[k].cf < marker_info[j].cf)
					k = j;
			}
		}
		if (k == -1) {
			object[i].visible = 0;
			continue;
		}

		/* calculate the transform for each marker */
		if (object[i].visible == 0) {
			arGetTransMat(&marker_info[k], object[i].marker_center,
					object[i].marker_width, object[i].trans);

		} else {
			arGetTransMatCont(&marker_info[k], object[i].trans,
					object[i].marker_center, object[i].marker_width,
					object[i].trans);
		}
		object[i].visible = 1;
	}

	arVideoCapNext();
	print_string_autor(autor);
	
	// IMPRIME MENSAJE EN VENTANA DE REPRODUCCION
	if (!comenzarCaptura){
		print_string(mensaje1);
	}
	if (comenzarCaptura) {
		print_string(mensaje2);
		/* draw the AR graphics */
		draw(object, objectnum);

		imprimir = 0;
		// La siguiente función captura los movimientos de la cabeza 
		// se le pasan los siguientes parametros:
		// object[0] -> base
		// object[3] -> head_front
		// object[1] -> head_right
		// object[2] -> head_left
		// head <- estructura de tipo Bone que contiene los angulos para el frame

		if (captureOption == 1 || captureOption == 2) {
			if (headMotion(object[0], object[3], object[1], object[2], &head)) {
				lastFrame.head = head;
				imprimir = 1;
			}
		}
		// La siguiente función captura los movimientos del brazo derecho
		// object[0] -> base
		// object[4] -> arm_right_front
		// object[5] -> arm_right_top
		// object[6] -> arm_right_botton
		// object[7] -> arm_right_back
		// arm_right <- estructura de tipo Bone que contiene los angulos para el frame
		if (captureOption == 1 || captureOption == 3) {
			if (armRightMotion(object[0], object[4], object[5], object[6],
					object[7], &arm_right)) {
				lastFrame.arm_right = arm_right;
				imprimir = 1;
			}
		}
		// La siguiente función captura los movimientos del antebrazo derecho
		// object[0] -> base
		// object[4] -> arm_right_front
		// object[5] -> arm_right_top
		// object[6] -> arm_right_botton
		// object[7] -> arm_right_back
		// object[8] -> arm_right_front
		// object[9] -> arm_right_top
		// object[10] -> arm_right_botton
		// object[11] -> arm_right_back
		// arm_right <- estructura de tipo Bone que contiene los angulos para el frame
		/*if (forearmRightMotion(object[4], object[5], object[6], object[7], object[8], object[9], object[10], object[11],
		 &forearm_right)) {
		 lastFrame.forearm_right = forearm_right;
		 imprimir = 1;
		 }
		 */

		//imprime en el archivo bvh el frame correspondiente
		if (imprimir) {
			printFrame();
		}
	}
	argSwapBuffers();

}

/* cleanup function called when program exits */
static void cleanup(void) {
	int win;
	arVideoCapStop();
	arVideoClose();
	argCleanup();
	win = glutGetWindow();
	if (win < 1) {
		glutSetWindow(1);
	}
	printf("Cerrando Ventana de Reproducción ");
	glutHideWindow();

}

/* draw the the AR objects */
static int draw(ObjectData_T *object, int objectnum) {
	int i;
	double gl_para[16];

	glClearDepth(1.0);
	glClear( GL_DEPTH_BUFFER_BIT);
	glEnable( GL_DEPTH_TEST);
	glDepthFunc( GL_LEQUAL);
	glEnable( GL_LIGHTING);

	/* calculate the viewing parameters - gl_para */
	for (i = 0; i < objectnum; i++) {
		if (object[i].visible == 0)
			continue;
		argConvGlpara(object[i].trans, gl_para);

		draw_object(object[i].id, gl_para);
	}

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);

	return (0);
}

/* draw the user object */
static int draw_object(int obj_id, double gl_para[16]) {
	GLfloat mat_ambient[] = { 0.0, 0.0, 1.0, 1.0 };
	GLfloat mat_ambient_collide[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat mat_flash[] = { 0.0, 0.0, 1.0, 1.0 };
	GLfloat mat_flash_collide[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat mat_flash_shiny[] = { 50.0 };
	GLfloat light_position[] = { 100.0, -200.0, 200.0, 0.0 };
	GLfloat ambi[] = { 0.1, 0.1, 0.1, 0.1 };
	GLfloat lightZeroColor[] = { 0.9, 0.9, 0.9, 0.1 };

	double modelMatrix[16];
	GLfloat m[16];

	argDrawMode3D();
	argDraw3dCamera(0, 0);
	glMatrixMode( GL_MODELVIEW);
	glLoadMatrixd(gl_para);

	// funcion que obtiene la matriz actual	 
	//glGetFloatv (GL_MODELVIEW_MATRIX, m);

	//imprimirMatrizOpengl(m);

	/* set the material */
	glEnable( GL_LIGHTING);
	glEnable( GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambi);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightZeroColor);

	glMaterialfv(GL_FRONT, GL_SHININESS, mat_flash_shiny);

	if (obj_id == 0) {
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash_collide);
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient_collide);

		glTranslatef(0.0, 0.0, 30.0);
		//draw_axis();
		glutSolidTeapot(20);

	} else if (obj_id == 1) {
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);

		glTranslatef(0.0, 0.0, 30.0);
		glutSolidTeapot(20);
		//draw_axis();
	} else if (obj_id == 2) {
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);

		glTranslatef(0.0, 0.0, 30.0);
		glutSolidTeapot(20);
		//draw_axis();
	} else {
		glMaterialfv(GL_FRONT, GL_SPECULAR, mat_flash);
		glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);

		glTranslatef(0.0, 0.0, 30.0);
		glutSolidTeapot(20);
		//draw_axis();

	}

	argDrawMode2D();

	return 0;
}

static void draw_axis(void) {
	glPushMatrix();
	glRotatef(90.0, 0.0, 1.0, 0.0);
	glColor4f(1.0, 0.0, 0.0, 1.0);
	glutSolidCone(5.0, 100.0, 20, 24);
	glPopMatrix();

	glPushMatrix();
	glRotatef(-90.0, 1.0, 0.0, 0.0);
	glColor4f(0.0, 1.0, 0.0, 1.0);
	glutSolidCone(5.0, 100.0, 20, 24);
	glPopMatrix();

	glPushMatrix();
	glRotatef(00.0, 0.0, 0.0, 1.0);
	glColor4f(0.0, 0.0, 1.0, 1.0);
	glutSolidCone(5.0, 100.0, 20, 24);
	glPopMatrix();

	return;
}

int multiplicarMatrices(double m[3][3], double n[3][3], double r[3][3]) {
	int i, j, k;
	for (k = 0; k < 3; ++k) {
		for (i = 0; i < 3; ++i) {
			for (j = 0; j < 3; ++j) {
				r[k][i] += m[k][j] * n[j][i];
			}
		}
	}
}

int imprimirMatriz(double m[3][3]) {
	int i, j;
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			printf("%8.4f ", m[i][j]);
		}
		printf("\n");
	}
	printf("\n");
}

int angulosToMatriz(double ax, double ay, double az, double rot[3][3]) {

	rot[0][0] = cos(ay) * cos(az);
	rot[0][1] = sin(ax) * sin(ay) * cos(az) - cos(ax) * sin(az);
	rot[0][2] = cos(ax) * sin(ay) * cos(az) + sin(ax) * sin(az);
	rot[1][0] = cos(ay) * sin(az);
	rot[1][1] = sin(ax) * sin(ay) * sin(az) + cos(ax) * cos(az);
	rot[1][2] = cos(ax) * sin(ay) * sin(az) - sin(ax) * cos(az);
	rot[2][0] = -sin(ay);
	rot[2][1] = sin(ax) * cos(ay);
	rot[2][2] = cos(ax) * cos(ay);

}

int imprimirMatrizOpengl(GLfloat m[16]) {

	printf("%8.4f ", m[0]);
	printf("%8.4f ", m[4]);
	printf("%8.4f ", m[8]);
	printf("%8.4f ", m[12]);
	printf("\n");
	printf("%8.4f ", m[1]);
	printf("%8.4f ", m[5]);
	printf("%8.4f ", m[9]);
	printf("%8.4f ", m[13]);
	printf("\n");
	printf("%8.4f ", m[2]);
	printf("%8.4f ", m[6]);
	printf("%8.4f ", m[10]);
	printf("%8.4f ", m[14]);
	printf("\n");
	printf("%8.4f ", m[3]);
	printf("%8.4f ", m[7]);
	printf("%8.4f ", m[11]);
	printf("%8.4f ", m[15]);
	printf("\n");
	printf("\n");

}

int matrizConversion(double mat[3][4], double rot[3][3]) {
	int i, j;
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			rot[i][j] = mat[i][j];
			//printf("%8.4f ", rot[i][j]);
		}
		//printf("\n");
	}
	return 0;
}

int obtenerAngulosRadianes(double rot[3][3], double *angx, double *angy,
		double *angz) {

	*angy = asin(-rot[2][0]);
	*angx = atan2(rot[2][1], rot[2][2]);
	*angz = atan2(rot[1][0], rot[0][0]);

	printf("%3.1f %3.1f %3.1f \n", *angx, *angy, *angz);
}

int obtenerAngulos(double rot[3][3], double *angx, double *angy, double *angz) {

	*angx = (atan2(rot[2][1], rot[2][2]) * 180) / PI;
	*angy = (asin(-rot[2][0]) * 180) / PI;
	*angz = (atan2(rot[1][0], rot[0][0]) * 180) / PI;

	printf("%3.1f %3.1f %3.1f \n", *angx, *angy, *angz);
}

int getAngulos(double rot[3][3], double *rfXAng, double *rfYAng, double *rfZAng) {
	// +-           -+   +-                                      -+
	// | r00 r01 r02 |   |  cy*cz  cz*sx*sy-cx*sz  cx*cz*sy+sx*sz |
	// | r10 r11 r12 | = |  cy*sz  cx*cz+sx*sy*sz -cz*sx+cx*sy*sz |
	// | r20 r21 r22 |   | -sy     cy*sx           cx*cy          |
	// +-           -+   +-                                      -+

	// gimbalLock == 0 , no se produce gimbal Lock
	// gimbalLock == 1 , se ha producido gimbal Lock por y angulo = pi/2
	// gimbalLock == 2 , se ha producido gimbal Lock por y angulo = -pi/2
	int gimbalLock = 0;

	if (rot[2][0] < 0.995) {
		if (rot[2][0] > -0.995) {
			// y_angulo = asin(-r20)
			// z_angulo = atan2(r10,r00)
			// x_angulo = atan2(r21,r22)
			*rfYAng = asin(-rot[2][0]);
			*rfZAng = atan2(rot[1][0], rot[0][0]);
			*rfXAng = atan2(rot[2][1], rot[2][2]);

		} else {
			// y_angulo = +pi/2
			// x_angulo + z_angulo = atan2(r01,r02)
			// NOTA.  La solución no es unica.  Tomamos x_angulo = 0.
			*rfYAng = PI / 2;
			*rfZAng = -atan2(rot[0][1], rot[0][2]);
			*rfXAng = 0.0;
			gimbalLock = 1;
		}
	} else {
		// y_angulo = -pi/2
		// x_angulo + z_angulo = atan2(-r01,-r02)
		// NOTA.  La solución no es unica.  Tomamos x_angulo = 0.
		*rfYAng = -PI / 2;
		*rfZAng = atan2(-rot[0][1], -rot[0][2]);
		*rfXAng = 0.0;
		gimbalLock = 2;

	}

	//printf("Radianes: %3.1f %3.1f %3.1f \n",*rfXAng,*rfYAng,*rfZAng);

	//printf("Centigrados: %3.1f %3.1f %3.1f \n",(*rfXAng*180)/PI, (*rfYAng*180)/PI, (*rfZAng*180)/PI);
	*rfXAng = (*rfXAng * 180) / PI;
	*rfYAng = (*rfYAng * 180) / PI;
	*rfZAng = (*rfZAng * 180) / PI - *rfYAng;

	return gimbalLock;
}

int copiarBaseBvh(void) {
	FILE *input, *output;
	char leido, copia[255];

	// Copia de archivo base
	printf("Ingrese el nombre del archivo nuevo\n");
	printf("Ejemplo : 'salida.bvh' (sin las comillas)\n");
	//scanf("%s",copia);

	if ((input = fopen("base.bvh", "r")) == NULL) {
		printf("\n El Archivo de entrada no existe \n");
		//system("pause");
		return 1;
	}

	output = fopen(copia, "w");

	while (!feof(input)) {
		leido = fgetc(input);
		if (!feof(input))
			fputc(leido, output);
	}
	printf("El archivo 'base.bvh' ha sido copiado a %s \n", copia);
	fclose(input);
	fclose(output);

}

int printFrame(void) {
	FILE *f;

	f = fopen("aux.bvh", "a");
	if (f == NULL) {
		//fprintf (stderr, "No se pudo acceder al fichero %s \n", copia);
		fprintf(stderr, "No se pudo acceder al fichero");
	} else {
		fprintf(
				f,
				"0.00 43.52 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 %3.2lf %3.2lf %3.2lf 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 %3.2lf %3.2lf %3.2lf %3.2lf %3.2lf %3.2lf 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00 \n",
				lastFrame.head.x, lastFrame.head.z, lastFrame.head.y,
				lastFrame.arm_right.z, lastFrame.arm_right.y,
				lastFrame.arm_right.x, lastFrame.forearm_right.y,
				lastFrame.forearm_right.z, lastFrame.forearm_right.x);
		cuenta_lineas++;
	}
	printf("numero de Lineas %d \n", cuenta_lineas);
	fclose(f);
	return 1;

}

int headMotion(ObjectData_T base, ObjectData_T head_front,
		ObjectData_T head_right, ObjectData_T head_left, Bone *head) {

	int imprimir;
	double cp;
	double wmat1[3][4], wmat2[3][4];
	double rot[3][3];
	double a, b, c;
	double cabezaX, cabezaY, cabezaZ;
	FILE *f;

	/* MOVIMIENTO DE LA CABEZA */
	imprimir = 0;
	/*comprobamos que es visible la marca base */
	if (base.visible > 0) {

		cp = 45.0;
		arUtilMatInv(base.trans, wmat1);
		/* ahora vamos describiendo los distintos casos de visualizacion de las marcas de la cabeza */
		//printf("objeto 0 :%d \n",object[0].visible);
		//printf("objeto 1 :%d \n",object[1].visible);
		//printf("objeto 2 :%d \n",object[2].visible);
		//printf("objeto 3 :%d \n\n",object[3].visible);
		/* solo se visualiza la marca frontal de la cabeza */
		if (head_front.visible > 0 && head_right.visible == 0
				&& head_left.visible == 0) {
			printf("Frontal\n");
			imprimir = 1;
			arUtilMatMul(wmat1, head_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &cabezaX, &cabezaY, &cabezaZ);
			cabezaX = cabezaX - cp;
			printf("%3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);

		} /* solo se visualiza la marca izquierda de la cabeza */
		else if (head_left.visible > 0 && head_right.visible == 0
				&& head_front.visible == 0) {
			printf("Izquierda\n");
			imprimir = 1;
			arUtilMatMul(wmat1, head_left.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			cabezaX = c;
			cabezaY = b - 90.0;
			cabezaZ = -a;
			printf("%3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);

		} /* solo se visualiza la marca derecha de la cabeza */
		else if (head_right.visible > 0 && head_front.visible == 0
				&& head_left.visible == 0) {
			printf("Derecha\n");
			imprimir = 1;
			arUtilMatMul(wmat1, head_right.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			cabezaX = -c;
			cabezaY = b + 90.0;
			cabezaZ = a;
			printf("%3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);

		} /* se visualizan las marcas frontal e izquierda de la cabeza */
		else if (head_front.visible > 0 && head_left.visible > 0
				&& head_right.visible == 0) {
			printf("Frontal, Izquierda\n");
			imprimir = 1;
			arUtilMatMul(wmat1, head_left.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			cabezaX = c;
			cabezaY = b - 90.0;
			cabezaZ = -a;
			printf("IZQ : %3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);
			arUtilMatMul(wmat1, head_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			printf("FRT : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			cabezaX = (a + cabezaX) / 2.0;
			cabezaY = (b + cabezaY) / 2.0;
			cabezaZ = (c + cabezaZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);

		} /* se visualizan las marcas frontal y derecha de la cabeza */
		else if (head_front.visible > 0 && head_right.visible > 0
				&& head_left.visible == 0) {
			printf("Frontal, Derecha\n");
			imprimir = 1;
			arUtilMatMul(wmat1, head_right.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			cabezaX = -c;
			cabezaY = b + 90.0;
			cabezaZ = a;
			printf("DER : %3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);
			arUtilMatMul(wmat1, head_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			printf("FRT : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			cabezaX = (a + cabezaX) / 2.0;
			cabezaY = (b + cabezaY) / 2.0;
			cabezaZ = (c + cabezaZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", cabezaX, cabezaY, cabezaZ);
		}
		(*head).x = cabezaX;
		(*head).y = cabezaY;
		(*head).z = cabezaZ;

		/*
		 if (imprimir) {
		 f = fopen("p1.bvh", "a");
		 if (f == NULL) {
		 //fprintf (stderr, "No se pudo acceder al fichero %s \n", copia);
		 fprintf(stderr, "No se pudo acceder al fichero");
		 } else {
		 fprintf(
		 f,
		 "0.000000 43.528519 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 %3.2lf  %3.2lf %3.2lf 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 0.000000 \n",
		 cabezaX, cabezaZ, cabezaY);

		 }
		 fclose(f);
		 }*/

	}
	return imprimir;

}

int armRightMotion(ObjectData_T base, ObjectData_T arm_right_front,
		ObjectData_T arm_right_top, ObjectData_T arm_right_botton,
		ObjectData_T arm_right_back, Bone *arm_right) {

	int imprimir;
	double cp;
	double wmat1[3][4], wmat2[3][4], wmat3[3][4], wmat4[3][4];
	double rot[3][3];
	double a, b, c;
	double brazoX, brazoY, brazoZ;
	FILE *f;
	Bone vacio;
	/* MOVIMIENTO DEL BRAZO */
	imprimir = 0;
	/*comprobamos que es visible la marca base */
	if (base.visible > 0) {

		// valor de correccion por si está inclinada hacia atrás
		cp = 0.0;
		arUtilMatInv(base.trans, wmat1);
		/* ahora vamos describiendo los distintos casos de visualizacion de las marcas del brazo*/

		/* solo se visualiza la marca frontal del brazo */
		if (arm_right_front.visible > 0 && arm_right_top.visible == 0
				&& arm_right_botton.visible == 0 && arm_right_back.visible == 0) {
			printf("Front\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			// angulos de la cara frontal , coincide con brazo final
			getAngulos(rot, &brazoX, &brazoY, &brazoZ);
			brazoX = brazoX - cp;
			printf("%3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
			// colocamos los valores en r_arm para usarlo en la siguient iteracion
			r_arm.front.x = brazoX;
			r_arm.front.y = brazoY;
			r_arm.front.z = brazoZ;
			r_arm.top = vacio;
			r_arm.botton = vacio;
			r_arm.back = vacio;

		} /* solo se visualiza la marca superior del brazo */
		else if (arm_right_top.visible > 0 && arm_right_front.visible == 0
				&& arm_right_botton.visible == 0 && arm_right_back.visible == 0) {
			printf("Top\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_top.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara superior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a + 90.0;
			brazoY = b;
			brazoZ = c;
			printf("%3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);

		} /* solo se visualiza la marca inferior del brazo */
		else if (arm_right_botton.visible > 0 && arm_right_top.visible == 0
				&& arm_right_front.visible == 0 && arm_right_back.visible == 0) {
			printf("Botton\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_botton.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara inferior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a - 90.0;
			brazoY = b;
			brazoZ = c;
			printf("%3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);

		} /* solo se visualiza la marca posterior del brazo */
		else if (arm_right_back.visible > 0 && arm_right_top.visible == 0
				&& arm_right_front.visible == 0 && arm_right_botton.visible
				== 0) {
			printf("Back\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_back.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara posterior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a - 180.0;
			brazoY = -b;
			brazoZ = -c;
			printf("%3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);

		} /* se visualizan las marcas frontal y superior del brazo */
		else if (arm_right_front.visible > 0 && arm_right_top.visible > 0
				&& arm_right_botton.visible == 0 && arm_right_back.visible == 0) {
			printf("Front, Top\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_top.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara superior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			printf("top original : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			//transformación a angulo del brazo
			brazoX = a + 90.0;
			brazoY = b;
			brazoZ = c;
			printf("TOP : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
			arUtilMatMul(wmat1, arm_right_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			printf("FRT : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			// calculo la media de las dos caras
			brazoX = (a + brazoX) / 2.0;
			brazoY = (b + brazoY) / 2.0;
			brazoZ = (c + brazoZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);

			// angulos entre las dos marcas
			arUtilMatInv(arm_right_front.trans, wmat3);
			arUtilMatMul(wmat3, arm_right_top.trans, wmat4);
			matrizConversion(wmat4, rot);
			getAngulos(rot, &a, &b, &c);
			printf("diferencia entre front y top : %3.2lf, %3.2lf, %3.2lf \n",
					a, b, c);

		} /* se visualizan las marcas frontal e inferior del brazo */
		else if (arm_right_front.visible > 0 && arm_right_botton.visible > 0
				&& arm_right_top.visible == 0 && arm_right_back.visible == 0) {
			printf("Front, Botton\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_botton.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara inferior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a - 90.0;
			brazoY = b;
			brazoZ = c;
			printf("BOTTON : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
			arUtilMatMul(wmat1, arm_right_front.trans, wmat2);
			matrizConversion(wmat2, rot);
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			printf("FRT : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			brazoX = (a + brazoX) / 2.0;
			brazoY = (b + brazoY) / 2.0;
			brazoZ = (c + brazoZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
		} /* se visualizan las marcas posterior y superior del brazo */
		else if (arm_right_back.visible > 0 && arm_right_top.visible > 0
				&& arm_right_botton.visible == 0 && arm_right_front.visible
				== 0) {
			printf("Back, Top\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_top.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara superior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a + 90.0;
			brazoY = b;
			brazoZ = c;
			printf("TOP : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
			arUtilMatMul(wmat1, arm_right_back.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara posterior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			a = a - 180.0;
			b = -b;
			c = -c;
			printf("BACK : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			brazoX = (a + brazoX) / 2.0;
			brazoY = (b + brazoY) / 2.0;
			brazoZ = (c + brazoZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
		} /* se visualizan las marcas posterior e inferior del brazo */
		else if (arm_right_back.visible > 0 && arm_right_botton.visible > 0
				&& arm_right_top.visible == 0 && arm_right_front.visible == 0) {
			printf("Back, Botton\n");
			imprimir = 1;
			arUtilMatMul(wmat1, arm_right_botton.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara inferior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			brazoX = a - 90.0;
			brazoY = b;
			brazoZ = c;
			printf("BOTTON : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
			arUtilMatMul(wmat1, arm_right_back.trans, wmat2);
			matrizConversion(wmat2, rot);
			//angulos de la cara posterior
			getAngulos(rot, &a, &b, &c);
			a = a - cp;
			//transformación a angulo del brazo
			a = a - 180.0;
			b = -b;
			c = -c;
			printf("BACK : %3.2lf, %3.2lf, %3.2lf \n", a, b, c);
			brazoX = (a + brazoX) / 2.0;
			brazoY = (b + brazoY) / 2.0;
			brazoZ = (c + brazoZ) / 2.0;
			printf("MED : %3.2lf, %3.2lf, %3.2lf \n", brazoX, brazoY, brazoZ);
		}
		(*arm_right).x = brazoX;
		(*arm_right).y = brazoY;
		(*arm_right).z = brazoZ;

	}
	return imprimir;

}

//TODO SOLO CUANDO LA MARCA FRONTAL DEL BRAZO ES VISIBLE
int forearmRightMotion(ObjectData_T arm_right_front,
		ObjectData_T arm_right_top, ObjectData_T arm_right_botton,
		ObjectData_T arm_right_back, ObjectData_T forearm_right_front,
		ObjectData_T forearm_right_top, ObjectData_T forearm_right_botton,
		ObjectData_T forearm_right_back, Bone *forearm_right) {
	/* MOVIMIENTO DEL ANTEBRAZO */
	int imprimir = 0;
	Bone antebrazo;
	if (arm_right_front.visible > 0) {
		if (armRightMotion(arm_right_front, forearm_right_front,
				forearm_right_top, forearm_right_botton, forearm_right_back,
				&antebrazo)) {
			*forearm_right = antebrazo;
			imprimir = 1;
		}

	} else if (arm_right_top.visible > 0) {

	} else if (arm_right_botton.visible > 0) {

	} else if (arm_right_back.visible > 0) {

	}

	return imprimir;

}

void print_string( char *string )
{
  int     i;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  /* display the position data */
  glTranslatef(-0.95, 0.920, 0.0);

  /* draw a white polygon */
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_POLYGON);
    glVertex2f(1.50, 0.10);
    glVertex2f(1.50, -0.02);
    glVertex2f(0.001, -0.02);
    glVertex2f(0.001, 0.10);
  glEnd();

  /* draw red text on the polygon */
  glColor3f(0.75, 0.0, 0.0);
  glRasterPos2i(0.0, 0.0);
  for (i=0; i<(int)strlen(string); i++) {
      if(string[i] != '\n' ) {
          glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, string[i]);
      }
      else {
          glTranslatef(0.0, -0.07, 0.0);
          glRasterPos2i(0.0, 0.0);
      }
  }

  return;
}

void print_string_autor( char *string )
{
  int     i;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();

  /* display the position data */
  glTranslatef(-0.95, -0.920, 0.0);

  /* draw a white polygon 
  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_POLYGON);
    glVertex2f(1.50, 0.10);
    glVertex2f(1.50, -0.12);
    glVertex2f(0.001, -0.12);
    glVertex2f(0.001, 0.10);
  glEnd();*/

  /* draw red text on the polygon */
  glColor3f(0.75, 0.0, 0.0);
  glRasterPos2i(0.0, 0.0);
  for (i=0; i<(int)strlen(string); i++) {
      if(string[i] != '\n' ) {
          glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, string[i]);
      }
      else {
          glTranslatef(0.0, -0.07, 0.0);
          glRasterPos2i(0.0, 0.0);
      }
  }

  return;
}

